using System;
using System.Collections.Generic;
using System.Text;

using Sensr;

namespace nac1
{
  class Program
  {
    static void Main(string[] args)
    {
      list_devices();
      list_device_capabilities();
      simple_stream();
    }

    static void list_devices()
    {
      ttr gp = new ttr();
      List<ttr.TTRDevice> ConnectedDevices;
      
      try
      {
        // Query API for the list of connected and supported devices.
        ConnectedDevices = gp.ListDevices();

        // Print out the list of devices that were found
        Console.WriteLine("Number of connected devices:{0}", ConnectedDevices.Count);
        foreach (ttr.TTRDevice dev in ConnectedDevices)
        {
          Console.WriteLine("{0}", dev.ToString());
        }
      }
      catch (ttr.TTR_Exception e)
      {
        // All of the GP functions will throw a TTR_Exception if an error occurs
        // you should always bracket GP access functions with a try catch block.
        Console.WriteLine("{0}", e);
      }
    }

    static void list_device_capabilities()
    {
      ttr gp = new ttr();
      ttr.DeviceCaps DevicesCaps;

      try
      {
        // Connect to first available device
        gp.Connect();

        // Query device for it's capabilities
        DevicesCaps = gp.GetDeviceCaps();
        Console.WriteLine("DeviceCaps: \n{0}", DevicesCaps.ToString());

        // Print out list of accelerometer ranges with index
        List<string> AccelRanges = DevicesCaps.AccelRanges();
        Console.WriteLine("Supported acceleration ranges:");
        for(int i = 0; i < AccelRanges.Count; i++)
        {
          Console.WriteLine("Range: {0} = {1}g", i, AccelRanges[i]);
        }
        Console.WriteLine();

        // Disconnect from device
        gp.Disconnect();
      }
      catch (ttr.TTR_Exception e)
      {
        // All of the GP functions will throw a TTR_Exception if an error occurs
        // you should always bracket GP access functions with a try catch block.
        Console.WriteLine("{0}", e);
      }
    }

    static void simple_stream()
    {
      ttr gp = new ttr();
      List<ttr.RawDataRecord> Samples;
      int Missed;
      ttr.DeviceIdentify DeviceID;

      try
      {
        // Connect to first available device, there are overloaded
        // versions of Connect that allow you to specify which device
        // to connect to.
        gp.Connect();

        // Call the Identify method to get device information such as serial number
        // and firmware version. It is not necessary to call this method after connect
        // I just included it to show you how.
        DeviceID = gp.Identify();
        Console.WriteLine("Information about connected device:");
        Console.WriteLine("Device Serial # = {0}", DeviceID.SerialNumber);
        Console.WriteLine("Device firmware version = {0}", DeviceID.FirmwareVersion.VersionString());
        Console.WriteLine();

        // Disables Monitoring(Epoch data processing), which you aren't 
        // using anyway, but just in case doesn't hurt to call it.
        gp.MonitorDisable();

        // Make sure the accelerometer is powered up
        gp.SetAccelSleep(false);

        // Set the accelerometer G range. The grange parameter is an index into a table
        // inside the unit. You can query the unit for what g ranges it supports
        // with the GetDeviceCaps method.
        gp.SetAccelRange(0);

        // Purge the internal sample buffer on the GP device.
        // The GP device contains a sample buffer that is large enough
        // to hold 1 second of data without loss, this eases the polling
        // requirement on the PC side.
        gp.AccelPurge();

        // Enable streaming mode in the device 
        gp.SampleStreamingEnable(true);

        Console.WriteLine("Streaming data:\nMissed : X : Y : Z");
        while (true)
        {
          // Get the samples from the GP. The GP will return the samples 
          // that are currently in the buffer, if any. The Missed parameter
          // indicates if any samples were missed due to buffer overflow.
          Samples = gp.GetSamples(out Missed);

          // Check to see if any samples were returned
          if (Samples.Count > 0)
          {
            // Print out the samples
            foreach(ttr.RawDataRecord samp in Samples)
            {
              // The fields within RawDataRecord that you will be most interested
              // in are X, Y, Z, and possibly Temperature. X, Y, and Z are in units of (g).
              // Temperature is in degrees C.
              Console.WriteLine("{0} : {1} : {2} : {3}", Missed, samp.X, samp.Y, samp.Z, samp.Temperature);
            }
          }

        }

      }
      catch (ttr.TTR_Exception e)
      {
        // All of the GP functions will throw a TTR_Exception if an error occurs
        // you should always bracket GP access functions with a try catch block.
        Console.WriteLine("{0}", e);
      }
      finally
      {
        // Disconnect when finished, not absolutely necessary, as the API will clean up
        // on shutdown, but always a good idea.
        gp.Disconnect();
      }
    }

  }
}
